#  aRT : API R - TerraLib                                                
#  Copyright (C) 2003-2008  LEG                                          
#                                                                        
#  This program is free software; you can redistribute it and/or modify  
#  it under the terms of the GNU General Public License as published by  
#  the Free Software Foundation; either version 2 of the License, or     
#  (at your option) any later version.                                   
#                                                                        
#  This program is distributed in the hope that it will be useful,       
#  but WITHOUT ANY WARRANTY; without even the implied warranty of        
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         
#  GNU General Public License for more details.                          
#                                                                        
#  You should have received a copy of the GNU Lesser General Public      
#  License along with this library.

# aRT.R: calls C++ functions of main.cpp.
# All this functions only check for user argument errors.
# Other errors are checked in the next layers.

.getConfig = function(file, key)
{
    data = readLines(file)
    key=paste("[", key, "]", sep="")

    i = 1
    while(data[i] != key && i <= length(data))
    {
        i = i + 1
    }
    if(i > length(data))
        stop("invalid key")
    i = i + 1

    result = list()
    while(substr(data[i],1,1) != "[" && i <= length(data))
    {
        d = unlist(strsplit(data[i], "\\="))
        result[[d[1]]] = d[2]

        i = i + 1
    }
    return(result)
}


setMethod("initialize", "aRTconn", function(.Object,
                                            user = system("echo $USER", TRUE),
									        password = "",
										    port = NULL,
										    host = "",
										    dbms = NULL,
										    name = NULL)
{
  if(!is.null(name))
  {
    conf = .getConfig("~/.aRTrc", name)

	for(i in 1:(length(conf)))
	{
      t = names(conf[i])
      d = conf[[i]]

           if(t == "user")     user = d
      else if(t == "password") password = d
      else if(t == "port")     port = as.integer(d)
      else if(t == "host")     host = d
      else if(t == "dbms")     dbms = d
	}

  }

  if(is.null(dbms))
  {
    if(FALSE)  { dbms = "sqlite"  }
    if(FALSE) { dbms = "postgre" }
    if(TRUE)   { dbms = "mysql"   }
  }

  if( missing(port) )
  {
    if(dbms == "mysql")   port = 3306
    if(dbms == "postgre") port = 5432
  }

  if(dbms == "mysql"  & !TRUE)   stop("MySQL DBMS not found in the installation")
  if(dbms == "posgre" & !FALSE) stop("PostgreSQL DBMS not found in the installation")
  if(dbms == "sqlite" & !FALSE)  stop("SQLite not found in the installation")
  
  .Object@pointer <- .Call("cppNewaRT", user, password, port, host, dbms, PACKAGE="aRT")
  reg.finalizer(.Object@pointer, .aRTremove)
  .Object
})

openConn = function(...) {
	new("aRTconn", ...)
}

setMethod("openDb", "aRTconn", function(object, dbname, ...) {
    sd = showDbs(object)
    if(length(sd) != 0 && !any(showDbs(object) == dbname)) # sqlite returns an empty sd
        stop("Database does not exist\n")
	new("aRTdb", object, dbname=dbname, ...)
})

setMethod("createDb", "aRTconn", function(object, dbname, delete=FALSE) {
	if(any(showDbs(object) == dbname))
    {
		if(delete) deleteDb(object, dbname, force=TRUE)
		else stop("Database already exists\n")
    }
	new("aRTdb", object, dbname=dbname, create=TRUE)
})

setMethod("getPermissions", "aRTconn", function(object, user="", global=TRUE) {
	.aRTcall(object, "cppGetPermissions", user=user, global=global)
})

setMethod("dropUser", "aRTconn", function(object, user, remote=FALSE, host="") {
	.aRTcall(object, "cppDropUser", user=user, remote=as(remote, "integer"), host=host)
	return (invisible())
})

setMethod("addPermission", "aRTconn", function(object, user, pass="", remote=FALSE, host="", privilege=c("all", "select", "insert", "create", "update"), db="*") {
	privilege=match.arg(privilege)
	.aRTcall(object, "cppAddPermission", user=user, pass=pass, remote=as(remote, "integer"), host=host, database=db, privilege=privilege)
	return (invisible())
})

setMethod("showDbs", "aRTconn",  function(object) { .aRTcall(object, "cppList")       })

setMethod("deleteDb", "aRTconn", function(object, dbname, force=FALSE) {
	.catSilent("Checking for database \'", dbname, "\' ... ", sep="")
	if(! any(showDbs(object) == dbname) )
	{
		.catSilent("no\n")
		warning(sprintf("Database '%s' does not exist.", dbname))
		return(invisible())
	}
	.catSilent("yes\n")

	if(.aRTconfirm(force))
		.aRTcall(object, "cppDeleteDb", dbname=dbname)
	return(invisible())
})

