# Example of spatial relations using the three geometric types
# This example uses data from Recife city, in Brazil
#
# Last change: 20080919 Pedro Ribeiro de Andrade

require(aRT)
aRTsilent(FALSE)

con = openConn()
con

if(any(showDbs(con)=="recife"))
  deleteDb(con, "recife", force=T)

db = createDb(con, "recife")

ldistricts = createLayer(db, "districts")
lpoints    = createLayer(db, "points")
lsewers    = createLayer(db, "sewers")

data(recife.districts)
data(recife.points)
data(recife.sewers)

addPolygons(ldistricts, recife.districts)
addPoints  (lpoints,    recife.points)
addLines   (lsewers,    recife.sewers)

tdistricts = createTable(ldistricts, "t_districts")
tpoints    = createTable(lpoints,    "t_points")
tsewers    = createTable(lsewers,    "t_sewers")

# remove R objects and create them again
# to show that the data are really stored in the database
rm(con)
rm(db)
invisible(gc())

con <- openConn()
bdrecife <- openDb(con, "recife", up=TRUE)
showLayers(bdrecife)

ldistricts = openLayer(bdrecife, "districts")
lpoints    = openLayer(bdrecife, "points")
lsewers    = openLayer(bdrecife, "sewers")

geodistricts = getPolygons(ldistricts)
geopoints    = getPoints(lpoints)
geosewers    = getLines(lsewers)

par(mfcol=c(2,2), mar=c(.5,.5,.5,.5))

# plotting districts and points
aRTplot(geodistricts, col="red")
aRTplot(geopoints, add=T)

# plotting sewers
aRTplot(geodistricts, col="gray", lty=0)
aRTplot(geosewers, col="blue", add=T)

## Spatial relations
# 1. districts which contain points
getRelation(ldistricts, "contains", lpoints)->rel1
rel1

# 2. touching neighbours of the districts which contain points
getRelation(ldistricts, "touch", id=rel1)->rel2

aRTplot(geodistricts, col="gray", lty=0)
aRTplot(geodistricts, col="yellow", id=rel1, add=T)
aRTplot(geodistricts, col="green", id=rel2, add=T)
# plot(geopoints, add=T)

# 3. sewers within districts which contain points
getRelation(lsewers, c("crosses","within"), layer=ldistricts, id=rel1)->rel3

aRTplot(geodistricts, col="gray", lty=0, id=rel1)
aRTplot(geodistricts, col="gray", lty=0, add=T)
aRTplot(geodistricts, col="yellow", lty=0, add=T, id=rel1)

aRTplot(geosewers, add=T)
aRTplot(geosewers, col="red", id=rel3, add=T)


